////////////////////////////////////////////////////////////////////
//
// File: 		procapi.c
// Project: 	enum16bit
//
// Desc:		This files contains much of the code for the
//				enum16bit application dealing with processes
//
// Revisions: 	Created 01/12/98
//
// Copyright(C) 1998, Tomas Restrepo. All rights reserved
//
///////////////////////////////////////////////////////////////////


#include <windows.h>
#include <windowsx.h>
#include <vdmdbg.h>
#include <tchar.h>
#include "procapi.h"
#include "psapi.h"

// Make sure some libraries get linked
#pragma comment (lib, "vdmdbg.lib")
#pragma comment (lib, "psapi.lib")




///////////////////////////////////////////////////////////////////
// Get32BitModuleName()
//
// Uses the PSAPI library to get the main module name
// for a 32-bit process. 
// returns the number of characters copied on the string
//
// NOTE: This is a useful function so have it handy
DWORD 
Get32BitModuleName (
		HANDLE	hProcess,
		LPTSTR	lpszBuffer,
		DWORD	dwBufferSize
		)
{
	HMODULE		hModules[1024];
	DWORD		dwSize = 0;

	// Get the list of modules in the process. We use a large (1024)
	// array of HMODULES for this task, although in reality
	// there won't be a need for so many.
	// It's possible, but because there's only 64 TLS slots
	// most process don't load more that 64 dlls
	if ( EnumProcessModules ( hProcess, hModules, 1024, &dwSize ) != 0 )
	{
		// Get the name for the first module,
		// which will be the name of the .exe launched
		if ( GetModuleBaseName ( hProcess, hModules[0], lpszBuffer,
					dwBufferSize * sizeof ( TCHAR ) ) != 0 )
			// succeeded. 
			return _tcslen ( lpszBuffer );
	}
	// there was a problem. GetLastError() will tell us the problem
	return 0L;

} // Get32BitModuleName()



///////////////////////////////////////////////////////////////////
// Enum16BitTasksProc()
//
// This is the enumeration callback for tasks.
// What we need to do here is get the task name.
// A 16bit task is a 16-bit app running under one of the NTVDM's subsystem
BOOL 
CALLBACK 
Enum16BitTasksProc (
		DWORD	dwThreadId, 
		WORD	hMod16, 
		WORD	hTask16, 
		PSZ		pszModName,		// contains the module name
		PSZ		pszFileName,	// full path of the exe
		LPARAM	lParam
		)
{
	TCHAR	lpszInfo[MAX_PATH + 15];
	// display this task
	wsprintf ( lpszInfo, _T("\t[%lu] %s"), dwThreadId, pszModName );
	ListBox_InsertString ( (HWND)lParam, -1, lpszInfo );

	
	return FALSE;
} // Enum16BitProc()



///////////////////////////////////////////////////////////////////
// Enum16BitProc()
//
// This enumeration callback calls VDMEnumTaskWOW()
// to get the tasks running in a particular wow
BOOL 
CALLBACK 
Enum16BitProc (
		DWORD	dwProcessId,
		DWORD	dwAttributes,
		LPARAM	lParam
		)
{
	TCHAR	lpszModuleName[MAX_PATH];
	TCHAR	lpszInfo[MAX_PATH + 15];
	HANDLE	hVDMProcess;

	// Open the process to get a handle to it
	// so we can get it's name
	hVDMProcess = OpenProcess ( PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, 
										FALSE, dwProcessId );
	if ( hVDMProcess == NULL )
	{
		TCHAR	string[250];
		wsprintf ( string, _T("Failed to open process with error: %lu"), GetLastError () );
		MessageBox ( NULL, string, _T("Enum16Proc"), MB_OK );
		return TRUE;		// end enumeration
	}

	// Get process main module name
	if ( Get32BitModuleName ( hVDMProcess, lpszModuleName, MAX_PATH ) == 0 )
		// failed to get name
		_tcscpy ( lpszModuleName, _T("Unknown") );

	wsprintf ( lpszInfo, _T("[%lu] %s"), dwProcessId, lpszModuleName );
	ListBox_InsertString ( (HWND)lParam, -1, lpszInfo );
	
	// Now, enumerate tasks within it
	// Notice we're using the undocumented VDMEnumTaskWOWEx()
	// which will allow us to get in one step the exe name
	// from the callback
	VDMEnumTaskWOWEx ( dwProcessId, Enum16BitTasksProc, lParam );

	// close process
	CloseHandle ( hVDMProcess );
	
	return FALSE;

} // Enum16BitProc()



///////////////////////////////////////////////////////////////////
// BuildProcessList()
//
// This function fills the listbox with the process list.
// The processes are all NTVDM subsystems, for which we need to 
// enumerate  tasks running inside them
// It also uses some helper calls
BOOL 
BuildProcessList (
		HWND hListBox
		)
{
	int		iNumProcesses;		// number of processes
	// clear the listbox
	ListBox_ResetContent (hListBox);

	// Now, do enumeration, passing the listbox handle
	// as an extra parameter to the enumeration
	// callback function
	iNumProcesses = VDMEnumProcessWOW ( Enum16BitProc, (LPARAM) hListBox );

	// if there are no processes running, inform the user
	if ( iNumProcesses == 0 )
		ListBox_AddString ( hListBox, _T("No Processes running") );


	return TRUE;
} // BuildProcessList()


