////////////////////////////////////////////////////////////////////
//
// File: 		enum16bit.c
// Project: 	enum16bit
//
// Desc:		This file presents a sample on how to
//				enumerate 16-bit processes under Windows NT
//
// Revisions: 	Created 30/11/98
//
// Copyright(C) 1998, Tomas Restrepo. All rights reserved
//
///////////////////////////////////////////////////////////////////

// Some includes and definitions
#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include "procapi.h"
#include "resource.h"

//
// HANDLE_DLGMSG macro
// allows handling dialog messages
//
#define HANDLE_DLGMSG(hwnd, msg, fn)								\
	case (msg): return SetDlgMsgResult (hwnd, msg,					\
		HANDLE_##msg((hwnd), (wParam), (lParam), (fn)))


///////////////////////////////////////////////////////////////////
// MainDlg_OnCommand()
//
// Handle command messages. 
// Close: End Dialog
// Refresh: Rebuild process list
void 
MainDlg_OnCommand (
		HWND	hwnd, 
		int		id, 
		HWND	hwndControl, 
		UINT	codeNotify
		)
{
	HWND	hListBox;
	switch (id)
	{
	case IDC_CLOSE:
		EndDialog (hwnd, 0);
		break;
		
	case IDC_REFRESH:
		hListBox = GetDlgItem ( hwnd, IDC_PROCESSLIST );
		if ( !BuildProcessList ( hListBox ) )
		{
			MessageBox ( hwnd, _T("Failed to build process list"), 
							_T("Enum16bit"), MB_OK );
		}
		break;
	}
} // MainDlg_OnCommand()


///////////////////////////////////////////////////////////////////
// MainDlg_OnInitDialog()
//
// Perform some initialiazation. First of all, we need to get the
// process list by calling the BuildProcessList() function
BOOL 
MainDlg_OnInitDialog (
		HWND	hwnd, 
		HWND	hwndFocus, 
		LPARAM	lParam
		)
{
	HWND hListBox = GetDlgItem ( hwnd, IDC_PROCESSLIST );

	if ( !BuildProcessList ( hListBox ) )
		// Something went wrong
		MessageBox ( hwnd, _T("Failed to build process list"), 
						_T("Enum16bit"), MB_OK );
	return TRUE;
} // MainDlg_OnInitDialog

//////////////////////////////////////////////////////////////////
// MainDlg_OnDestroy()
//
// Clean up
void 
MainDlg_OnDestroy (
		HWND hwnd
		)
{
	// TODO: Clean up

} // MainDlg_OnDestroy


///////////////////////////////////////////////////////////////////
// MainDlgProc ()
// 
// Main dialog window procedure. Remember we're using the message
// crackers in windowsx.h
BOOL 
CALLBACK 
MainDlgProc (
		HWND	hwnd, 
		UINT	msg, 
		WPARAM	wParam, 
		LPARAM	lParam
		)
{
	switch (msg)
	{
		HANDLE_DLGMSG (hwnd, WM_INITDIALOG,	MainDlg_OnInitDialog);
		HANDLE_DLGMSG (hwnd, WM_DESTROY,	MainDlg_OnDestroy);
		HANDLE_DLGMSG (hwnd, WM_COMMAND,	MainDlg_OnCommand);
	}
	return FALSE;
	
} // MainDlgProc



///////////////////////////////////////////////////////////////////
// _tWinMain ()
//
// Entry Point. Just create the main dialog box
int 
WINAPI 
_tWinMain (
	  HINSTANCE hInstance, 
	  HINSTANCE hPrevInstance, 
	  LPTSTR	lpszCmdLine, 
	  int		nCmdShow
	  )
{

	DialogBox (hInstance, MAKEINTRESOURCE (IDD_MAINDLG), NULL, MainDlgProc);
	return 0;

} // _tWinMain


